<?php
/*
Plugin Name: Twitter Post
Description: Creates a CMS section that automatically posts to twitter.  You must authorize this plugin.
Version: 2.05
Requires at least: 2.53
*/

// DON'T UPDATE ANYTHING BELOW THIS LINE

addAction('admin_postlogin',               'twitterpost_dispatch',  null, 0);
addAction('section_init',                  'twitterpost_upgradeIfNeeded', null, 2);
addAction('section_init',                  'twitterpost_errorChecking', null, 1);
addAction('plugin_actions',                'twitterpost_addPluginAction');
addAction('record_preedit',                'twitterpost_disableModify', null, 1);
addAction('record_preerase',               'twitterpost_disableErase',  null, 1);
addAction('record_save_posterrorchecking', 'twitterpost_postTweet',     null, 1);
addFilter('record_saved_message',          'twitterpost_saveMessage',   null, 1);

$GLOBALS['TWITTERPOST_TABLENAME'] = '_twittermanager';  // CMS tablename tweets are created in
$GLOBALS['TWITTERPOST_FIELDNAME'] = 'tweet';            // CMS fieldname with tweet content

$GLOBALS['TWITTERPOST_CONSUMER_SECRET'] = "29VTPdERupkDj404kq8SxjpBsNlPtZDA7jbpMk0Bs8";
$GLOBALS['TWITTERPOST_CONSUMER_KEY']    = "F8VuGSTuc5kMqlfHg14w";

//
function twitterpost_errorChecking($tableName) {
  $isPluginMenu  = ($tableName == 'admin' && @$_REQUEST['action'] == 'plugins');
  $isTwitterMenu = ($tableName == $GLOBALS['TWITTERPOST_TABLENAME']);

  // Twitter Section: Display "Not authorized" error
  if ($isTwitterMenu) {
    $access_data = _twitterpost_load_access_data();
    if (empty($access_data)) {
      alert("Please authorize access to your Twitter account in Admin > Plugins > Twitter Post > Authorize.<br/>\n");
    }
    return;
  }

}

//
function twitterpost_upgradeIfNeeded($tableName) {
  $isPluginMenu  = ($tableName == 'admin' && @$_REQUEST['action'] == 'plugins');
  $isTwitterMenu = ($tableName == $GLOBALS['TWITTERPOST_TABLENAME']);

  if (!$isPluginMenu && !$isTwitterMenu) { return; }

  // load plugin settings data
  $pluginSettings = _twitterpost_load_access_data();

  ### upgrade to 2.02
  if (@$pluginSettings['plugin_version'] < 2.02) {

    // bug fix: <2.02 set $schema['_requiredPlugins'] to twitterPost.php, which caused the
    // ... plugin not to work if it was in a subfolder such as /plugins/twitterPost/
    // ... so we're updating the schema value for _requiredPlugins for users who are upgrading
    list($thisPluginPath, $thisPluginUrl) = getPluginPathAndUrl();

    $schema = loadSchema($GLOBALS['TWITTERPOST_TABLENAME']);
    $schema['_requiredPlugins'] = $thisPluginPath;
    saveSchema($GLOBALS['TWITTERPOST_TABLENAME'], $schema);

    if ($isTwitterMenu) { $GLOBALS['schema'] = $schema; } // update global schema so we don't get the plugin error


    // update version
    $pluginSettings['plugin_version'] = '2.02';
    _twitterpost_save_access_data($pluginSettings);
  }






}

// on plugin_actions: display an "Authorize" link
function twitterpost_addPluginAction($pluginPath) {
  list($thisPluginPath, $thisPluginUrl) = getPluginPathAndUrl();
  if ($thisPluginPath != $pluginPath) { return; }

  echo "<a href='?_pluginName=twitterpost&amp;_pluginAction=authorize'>Authorize</a><br />";
}

//
function _twitterpost_load_access_data() {
  $filepath = dirname(__FILE__) . '/access_data.dat.php';
  $data     = loadStructOrINI($filepath);
  return $data;
}

function _twitterpost_save_access_data($data) {
  return saveStruct(dirname(__FILE__) . '/access_data.dat.php', $data);
}

// NOTE: Tweets get posted as: 3 minutes ago from API
// To change "from API" to an application name can be done but requires a lot of work
// and a change of the authentication method.  See: http://apiwiki.twitter.com/FAQ#HowdoIget“fromMyApp”appendedtoupdatessentfrommyAPIapplication
function twitterpost_postTweet($tableName) {
  if ($tableName != $GLOBALS['TWITTERPOST_TABLENAME']) { return; }

  $status = @$_REQUEST[ $GLOBALS['TWITTERPOST_FIELDNAME'] ];

  // Post message to twitter
  $access_data = _twitterpost_load_access_data();
  $response = _twitterpost_oauth_request('https://api.twitter.com/1.1/statuses/update.json', $access_data['oauth_access_token_secret'], array(
    'status'      => $status,
    'oauth_token' => $access_data['oauth_access_token'],
  ));

  $response = json_decode($response, true);

  // display twitter errors
  $errors = '';
  if     (@$response['error']) { $errors .= "Twitter Error: {$response['error']}\n"; }
  elseif (!@$response['id'])   { $errors .= "Twitter Error: No twitter post 'id' was returned for the last tweet!\n"; }
  if ($errors) { // errors will be display by ajax form
    if (@$response['error'] == "Invalid / expired Token") { $errors .= "\nTry Authenticating again under Admin > Plugins > Twitter Post > Authenticate."; }
    die($errors);
  }
}

//
function twitterpost_saveMessage($message) {
  if ($GLOBALS['tableName'] != $GLOBALS['TWITTERPOST_TABLENAME']) { return $message; }
  return "Tweet posted!";
}

//
function twitterpost_disableModify($tableName) {
  if ($tableName != $GLOBALS['TWITTERPOST_TABLENAME']) { return; }
  if (!@$_REQUEST['num']) { return; } // skip if adding record

  alert("Can't modify tweets! Tweets can only be removed and reposted on twitter directly!<br/>\n");
  include('lib/menus/default/list.php');
  exit;
}

//
function twitterpost_disableErase($tableName) {
  if ($tableName != $GLOBALS['TWITTERPOST_TABLENAME']) { return; }

  alert("Can't erase tweets! Tweets must be removed from twitter directly!<br/>\n");
  include('lib/menus/default/list.php');
  exit;
}

function twitterpost_dispatch() {
  twitterpost_createSchema();

  if (!$GLOBALS['CURRENT_USER']['isAdmin'])       { return; } // only allow admin access
  if (@$_REQUEST['_pluginName'] != 'twitterpost') { return; } // only run for this plugin

  // dispatch plugin actions
  $action = @$_REQUEST['_pluginAction'];
  if     ($action == 'authorize') { twitterpost_authorize(); }
  elseif ($action == 'callback')  { twitterpost_authorize_callback(); }
  else                            { die(__FUNCTION__ . ": Unknown action"); }

  //
  exit;
}

//
function twitterpost_authorize() {
  $request_token = _twitterpost_oauth_request('https://api.twitter.com/oauth/request_token', null, array(
    'oauth_callback' => thisPageUrl(array('_pluginAction' => 'callback')),
  ));
  parse_str($request_token, $request_token);
  $_SESSION['oauth_token_secret'] = $request_token['oauth_token_secret'];
  redirectBrowserToURL("https://api.twitter.com/oauth/authorize?oauth_token=".urlencode($request_token['oauth_token']));
}

//
function twitterpost_authorize_callback() {
  $token_secret = $_SESSION['oauth_token_secret'];
  if (!$token_secret) { die("oauth_token_secret not found in session data"); }
  $access_token = _twitterpost_oauth_request('https://api.twitter.com/oauth/access_token', $token_secret, array(
    'oauth_token'     => $_REQUEST['oauth_token'],
    'oauth_verifier'  => $_REQUEST['oauth_verifier'],
  ));
  parse_str($access_token, $access_token);
  if (@$access_token['oauth_token'] && @$access_token['oauth_token_secret']) {
    _twitterpost_save_access_data(array(
      'oauth_access_token'        => $access_token['oauth_token'],
      'oauth_access_token_secret' => $access_token['oauth_token_secret'],
    ));
    notice("Authorization successful! You can now use Twitter Post!");
  }
  else {
    alert("Authorization failed! Access Token was not returned! Response was ".print_r($access_token, true)."<br/>\n");
  }
  showInterface('admin/plugins.php');
}

//
function twitterpost_createSchema() {

  //
  $schemaDir  = realpath(DATA_DIR . '/schema');
  $schemaPath = "$schemaDir/{$GLOBALS['TWITTERPOST_TABLENAME']}.ini.php";
  if (file_exists($schemaPath)) { return; }

  list($pluginPath) = getPluginPathAndUrl();

  // create schema
  $schema = array(
    'menuName'             => 'Twitter',
    'menuType'             => 'multi',
    'menuOrder'            => time(),
    'listPageFields'       => 'createdDate, createdBy.fullname, tweet',
    'listPageOrder'        => 'createdDate DESC',
    'listPageSearchFields' => 'tweet',
    '_disableAdd'          => 0,
    '_disableErase'        => 1,
    '_disableModify'       => 1,
    '_requiredPlugins'     => basename($pluginPath),

    'num'              => array('order' => '1', 'type' => 'none',      'label' => 'Record Number', 'isSystemField' => '1', ),
    'createdDate'      => array('order' => '2', 'type' => 'none',      'label' => 'Created',       'isSystemField' => '1', ),
    'createdByUserNum' => array('order' => '3', 'type' => 'none',      'label' => 'Created By',    'isSystemField' => '1', ),
    'tweet'            => array('order' => '7', 'type' => 'textfield', 'label' => 'Tweet',         'isSystemField' => '0',
                                'fieldWidth'       => '500',
                                'isRequired'       => '1',
                                'maxLength'        => '140',
                                'customColumnType' => 'VARCHAR(255)', ),
  );
  saveSchema($GLOBALS['TWITTERPOST_TABLENAME'], $schema);
  notice("Created new 'Twitter' section<br/>\n");
  createMissingSchemaTablesAndFields();

}

function _twitterpost_oauth_request($url, $token_secret = null, $params) {
  // add generic elements to params
  $params = array_merge(array(
    'oauth_consumer_key'     => $GLOBALS['TWITTERPOST_CONSUMER_KEY'],
    'oauth_nonce'            => uniqid(''),
    'oauth_signature_method' => 'HMAC-SHA1',
    'oauth_timestamp'        => time(),
    'oauth_version'          => '1.0',
  ), $params);

  // construct signature
  $encoded_params = array_combine(_twitterpost_urlencode_rfc3986(array_keys($params)), _twitterpost_urlencode_rfc3986(array_values($params)));
  uksort($encoded_params, 'strcmp');
  $serialize_post_element = create_function('$k, $v', 'return $k.\'=\'.$v;');
  $normalized_params = implode('&', array_map($serialize_post_element, array_keys($encoded_params), array_values($encoded_params)));
  $signature_base_string = 'POST&'._twitterpost_urlencode_rfc3986($url).'&'._twitterpost_urlencode_rfc3986($normalized_params);
  $signature = base64_encode(_twitterpost_hmacsha1($signature_base_string, _twitterpost_urlencode_rfc3986($GLOBALS['TWITTERPOST_CONSUMER_SECRET']) . '&' . _twitterpost_urlencode_rfc3986(@$token_secret)));

  // construct Authorization header
  $encoded_params['oauth_signature'] = _twitterpost_urlencode_rfc3986($signature);
  $encoded_header_params = array();
  foreach ($encoded_params as $k => $v) { if (preg_match('/^oauth_/', $k)) { $encoded_header_params[$k] = $v; } } // filter keys /^oauth_/
  $serialize_oauth_header_element = create_function('$k, $v', 'return $k.\'="\'.$v.\'"\';');
  $oauth_header_data = implode(", ", array_map($serialize_oauth_header_element, array_keys($encoded_header_params), array_values($encoded_header_params)));
  $headers = array('Authorization' => 'OAuth ' . $oauth_header_data);

  // construct POST content
  $encoded_post_params = array();
  foreach ($encoded_params as $k => $v) { if (!preg_match('/^oauth_/', $k)) { $encoded_post_params[$k] = $v; } } // filter keys !/^oauth_/
  $query = implode("&", array_map($serialize_post_element, array_keys($encoded_post_params), array_values($encoded_post_params)));
  if ($query) { $query = '?' . $query; }

  // make request
  list($response) = getPage($url . $query, 5, $headers, true);
  if (!$response) { die("Could not establish connection to $url."); }
  return $response;
}

function _twitterpost_urlencode_rfc3986($input) {
  if (is_array($input)) {
    return array_map('_twitterpost_urlencode_rfc3986', $input);
  }
  else if (is_scalar($input)) {
    return str_replace('+',' ',str_replace('%7E', '~', rawurlencode($input)));
  }
  else{
    return '';
  }
}

function _twitterpost_hmacsha1($data, $key) {
  if (strlen($key) > 64) { $key = pack('H*', sha1($key)); }
  $key  = str_pad($key, 64, chr(0x00));
  $ipad = str_repeat(chr(0x36), 64);
  $opad = str_repeat(chr(0x5c), 64);
  $hmac = pack('H*',sha1(($key^$opad).pack('H*',sha1(($key^$ipad).$data))));
  return $hmac;
}

?>
